-- @release $Id: Uptime.lua 1 2016-08-06 02:50:07Z reighnman $

if not ShamAndAwe then return end

local L = LibStub("AceLocale-3.0"):GetLocale("ShamAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
local C = ShamAndAwe.constants -- Defined in Constants.LUA no locale needed.

ShamAndAwe.UptimeFrame = CreateFrame("Frame", "SAA_UptimeFrame", UIParent)

---------------------------
-- Uptime functions
----------------------------

function ShamAndAwe:InitialiseUptime()
  ShamAndAwe.uptime = {}
  ShamAndAwe.uptime = {
    session = {},
    lastfight = {},
    incombat = false,
    TimerEvent = nil,
    currentTime = nil,
  }
  self:InitialiseUptimeBuffs(ShamAndAwe.uptime.session)
  self:InitialiseUptimeBuffs(ShamAndAwe.uptime.lastfight)
end

function ShamAndAwe:InitialiseUptimeBuffs(timeframe)
  timeframe.totalTime = 0
  if not timeframe.buffs then
    timeframe.buffs = {}
  end

  --Stormlash
  if not timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])] then
    timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])] = {}
    timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].name = GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])
    timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].icon = ShamAndAwe.constants["Stormlash Icon"]
  end
  timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].uptime = 0

  --[[
  --Ignore Pain
  if not timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])] then
    timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])] = {}
    timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].name = GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])
    timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].icon = ShamAndAwe.constants["Ignore Pain Icon"]
  end
  timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].uptime = 0

  --Shield Block
  if not timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])] then
    timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])] = {}
    timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].name = GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])
    timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].icon = ShamAndAwe.constants["Shield Block Icon"]
  end
  timeframe.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].uptime = 0
  ]]
end

function ShamAndAwe:UpdateUptime()
  if not ShamAndAwe.uptime.incombat then
    return
  end
  local currentTime = GetTime()
  -- add conditional safety measure - we should never have no current time but to be sure...
  if not ShamAndAwe.uptime.currentTime then
    ShamAndAwe.uptime.currentTime = currentTime
  end
  local diffTime = currentTime - ShamAndAwe.uptime.currentTime

  ShamAndAwe.uptime.session.totalTime = ShamAndAwe.uptime.session.totalTime + diffTime;
  ShamAndAwe.uptime.lastfight.totalTime = ShamAndAwe.uptime.lastfight.totalTime + diffTime;

  local i = 1
  local buff = UnitBuff("player", i)
  while buff do
    if ShamAndAwe.uptime.session.buffs[buff] then
      ShamAndAwe.uptime.session.buffs[buff].uptime = ShamAndAwe.uptime.session.buffs[buff].uptime + diffTime
    end
    if ShamAndAwe.uptime.lastfight.buffs[buff] then
      ShamAndAwe.uptime.lastfight.buffs[buff].uptime = ShamAndAwe.uptime.lastfight.buffs[buff].uptime + diffTime
    end
    i = i + 1
    buff = UnitBuff("player", i)
  end
  ShamAndAwe.uptime.currentTime = currentTime
  self:UpdateUptimeFrames(false)
end

function ShamAndAwe:CreateUptimeFrame()
  ShamAndAwe:SetSpecialization()
  ShamAndAwe.UptimeFrame:ClearAllPoints()
  ShamAndAwe.UptimeFrame:SetScale(ShamAndAwe.db.char.uptime.scale)
  ShamAndAwe.UptimeFrame:SetFrameStrata("BACKGROUND")
  ShamAndAwe.UptimeFrame:SetWidth(ShamAndAwe.db.char.uptime.fWidth + ShamAndAwe.db.char.uptime.barHeight)
  ShamAndAwe.UptimeFrame:SetHeight(ShamAndAwe.db.char.uptime.fHeight + 16)
  ShamAndAwe.UptimeFrame:SetBackdrop(self.frameBackdrop)
  ShamAndAwe.UptimeFrame:SetBackdropColor(0, 0, 0, ShamAndAwe.db.char.uptime.alpha)
  ShamAndAwe.UptimeFrame:SetMovable(true)
  ShamAndAwe.UptimeFrame:RegisterForDrag("LeftButton")
  ShamAndAwe.UptimeFrame:SetPoint(ShamAndAwe.db.char.uptime.point, ShamAndAwe.db.char.uptime.relativeTo, ShamAndAwe.db.char.uptime.relativePoint, ShamAndAwe.db.char.uptime.xOffset, ShamAndAwe.db.char.uptime.yOffset)
  ShamAndAwe.UptimeFrame:SetScript("OnDragStart",
    function()
      ShamAndAwe.UptimeFrame:StartMoving();
    end );
  ShamAndAwe.UptimeFrame:SetScript("OnDragStop",
    function()
      ShamAndAwe.UptimeFrame:StopMovingOrSizing();
      ShamAndAwe.UptimeFrame:SetScript("OnUpdate", nil);
      self:FinishedMoving(ShamAndAwe.db.char.uptime, ShamAndAwe.UptimeFrame);
    end );

  local barfont = media:Fetch("font", ShamAndAwe.db.char.barfont)
  if not ShamAndAwe.UptimeFrame.topText then
    ShamAndAwe.UptimeFrame.topText = ShamAndAwe.UptimeFrame:CreateFontString(nil, "OVERLAY")
  end
  ShamAndAwe.UptimeFrame.topText:SetTextColor(1,1,1,1)
  ShamAndAwe.UptimeFrame.topText:SetFont(barfont, ShamAndAwe.db.char.barfontsize, ShamAndAwe.db.char.barfonteffect)
  ShamAndAwe.UptimeFrame.topText:SetPoint("TOP", ShamAndAwe.UptimeFrame, "TOP", 0, - 2)
  ShamAndAwe.UptimeFrame.topText:SetText(L["uptime_session"])

  if not ShamAndAwe.UptimeFrame.ResetButton then
    ShamAndAwe.UptimeFrame.ResetButton=CreateFrame("Button", nil, ShamAndAwe.UptimeFrame)
  end
  local size = ShamAndAwe.UptimeFrame.topText:GetHeight()
  ShamAndAwe.UptimeFrame.ResetButton:SetNormalTexture("Interface\\Addons\\ShamAndAwe\\textures\\icon-reset")
  ShamAndAwe.UptimeFrame.ResetButton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight.blp")
  ShamAndAwe.UptimeFrame.ResetButton:SetWidth(size)
  ShamAndAwe.UptimeFrame.ResetButton:SetHeight(size)
  ShamAndAwe.UptimeFrame.ResetButton:SetPoint("TOPRIGHT", ShamAndAwe.UptimeFrame.topText, "TOPLEFT", -5, 0)
  ShamAndAwe.UptimeFrame.ResetButton:SetScript("OnClick",function() ShamAndAwe:ResetUptimeValues() end)
  ShamAndAwe.UptimeFrame.ResetButton:SetFrameLevel(ShamAndAwe.UptimeFrame.ResetButton:GetFrameLevel()+1)
  ShamAndAwe.UptimeFrame.ResetButton:SetScript("OnEnter", function() UptimeResetButtonTooltip() end);
  ShamAndAwe.UptimeFrame.ResetButton:SetScript("OnLeave", function() GameTooltip:Hide() end);

  local baseOffset = (-1 * ShamAndAwe.db.char.uptime.fHeight / 6) - 2 -- 6 bars + top line + middle line
  if not ShamAndAwe.UptimeFrame.middleText then
    ShamAndAwe.UptimeFrame.middleText = ShamAndAwe.UptimeFrame:CreateFontString(nil, "OVERLAY")
  end
  ShamAndAwe.UptimeFrame.middleText:SetTextColor(1,1,1,1)
  ShamAndAwe.UptimeFrame.middleText:SetFont(barfont, ShamAndAwe.db.char.barfontsize, ShamAndAwe.db.char.barfonteffect)
  ShamAndAwe.UptimeFrame.middleText:SetPoint("TOP", ShamAndAwe.UptimeFrame, "TOP", 0, 3 * baseOffset - 2)
  ShamAndAwe.UptimeFrame.middleText:SetText(L["uptime_lastfight"])
  if ShamAndAwe.db.char.uptime.show and not ShamAndAwe.db.char.disabled then
    ShamAndAwe.UptimeFrame:Show()
  else
    ShamAndAwe.UptimeFrame:Hide()
  end
  local session = ShamAndAwe.uptime.session
  local lastfight = ShamAndAwe.uptime.lastfight

  --Enhancement
  self:CreateUptimeBarFrame(session.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])], ShamAndAwe.db.char.uptime.sl, baseOffset)
  self:CreateUptimeBarFrame(lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])], ShamAndAwe.db.char.uptime.sl, 4 * baseOffset)
  -- if ShamAndAwe.db.char.currentspec == "Enhancement" then
  ShamAndAwe.uptime.session.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame:Show()
  ShamAndAwe.uptime.lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame:Show()

  -- else
  -- ShamAndAwe.uptime.session.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame:Hide()
  -- ShamAndAwe.uptime.lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame:Hide()
  -- end
  --[[
  --Restoration
  self:CreateUptimeBarFrame(session.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])], ShamAndAwe.db.char.uptime.ip, baseOffset)
  self:CreateUptimeBarFrame(lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])], ShamAndAwe.db.char.uptime.ip, 4 * baseOffset)

  self:CreateUptimeBarFrame(session.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])], ShamAndAwe.db.char.uptime.sb, 2 * baseOffset)
  self:CreateUptimeBarFrame(lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])], ShamAndAwe.db.char.uptime.sb, 5 * baseOffset)

  if ShamAndAwe.db.char.currentspec == "Restoration" then
    ShamAndAwe.uptime.session.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame:Show()
    ShamAndAwe.uptime.lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame:Show()

    ShamAndAwe.uptime.session.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame:Show()
    ShamAndAwe.uptime.lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame:Show()
  else
    ShamAndAwe.uptime.session.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame:Hide()
    ShamAndAwe.uptime.lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame:Hide()

    ShamAndAwe.uptime.session.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame:Hide()
    ShamAndAwe.uptime.lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame:Hide()
  end
  ]]
  ShamAndAwe:ResetUptimeValues()
end

function ShamAndAwe:CreateUptimeBarFrame(buff, buffColors, frameOffset)
  if not buff.barFrame then
    buff.barFrame = CreateFrame("Frame", nil, ShamAndAwe.UptimeFrame)
  end

  buff.barFrame:SetFrameStrata("LOW")
  buff.barFrame:SetWidth(ShamAndAwe.db.char.uptime.barWidth + ShamAndAwe.db.char.uptime.barHeight)
  buff.barFrame:SetHeight(ShamAndAwe.db.char.uptime.barHeight)
  buff.barFrame:ClearAllPoints()
  buff.barFrame:SetPoint("TOPLEFT", ShamAndAwe.UptimeFrame, "TOPLEFT", ShamAndAwe.db.char.uptime.barHeight, frameOffset)
  buff.barFrame:SetScript("OnUpdate", function() self:UpdateUptimeFrames(false); end );

  if not buff.barFrame.buffIcon then
    buff.barFrame.buffIcon = CreateFrame("Frame", nil, buff.barFrame)
  end
  buff.barFrame.buffIcon:SetWidth(ShamAndAwe.db.char.uptime.barHeight)
  buff.barFrame.buffIcon:SetHeight(ShamAndAwe.db.char.uptime.barHeight)
  buff.barFrame.buffIcon:SetBackdrop({ bgFile = buff.icon })
  buff.barFrame.buffIcon:SetPoint("TOPRIGHT", buff.barFrame, "TOPLEFT", 12, 0)

  if not buff.barFrame.statusbar then
    buff.barFrame.statusbar = CreateFrame("StatusBar", nil, buff.barFrame, "TextStatusBar")
  end
  buff.barFrame.statusbar:ClearAllPoints()
  buff.barFrame.statusbar:SetHeight(ShamAndAwe.db.char.uptime.barHeight)
  buff.barFrame.statusbar:SetWidth(ShamAndAwe.db.char.uptime.barWidth)
  buff.barFrame.statusbar:SetPoint("RIGHT", buff.barFrame, "RIGHT")
  buff.barFrame.statusbar:SetStatusBarTexture(media:Fetch("statusbar", ShamAndAwe.db.char.texture))
  buff.barFrame.statusbar:SetStatusBarColor(buffColors.r, buffColors.g, buffColors.b, buffColors.a)
  buff.barFrame.statusbar:SetMinMaxValues(0,1)
  buff.barFrame.statusbar:SetValue(0)

  local barfont = media:Fetch("font", ShamAndAwe.db.char.barfont)
  if not buff.barFrame.statusbar.leftText then
    buff.barFrame.statusbar.leftText = buff.barFrame.statusbar:CreateFontString(nil, "OVERLAY")
  end
  buff.barFrame.statusbar.leftText:SetTextColor(1,1,1,1)
  buff.barFrame.statusbar.leftText:SetFont(barfont, ShamAndAwe.db.char.barfontsize, ShamAndAwe.db.char.barfonteffect)
  buff.barFrame.statusbar.leftText:SetPoint("TOPLEFT", buff.barFrame.statusbar, "TOPLEFT")
  buff.barFrame.statusbar.leftText:SetText("0.0s")

  if not buff.barFrame.statusbar.rightText then
    buff.barFrame.statusbar.rightText = buff.barFrame.statusbar:CreateFontString(nil, "OVERLAY")
  end
  buff.barFrame.statusbar.rightText:SetTextColor(1,1,1,1)
  buff.barFrame.statusbar.rightText:SetFont(barfont, ShamAndAwe.db.char.barfontsize, ShamAndAwe.db.char.barfonteffect)
  buff.barFrame.statusbar.rightText:SetPoint("TOPRIGHT", buff.barFrame.statusbar, "TOPRIGHT")
  buff.barFrame.statusbar.rightText:SetText("0%")

  buff.barFrame.statusbar:SetScript("OnEnter", function() UptimeStatusBarTooltip(buff) end);
  buff.barFrame.statusbar:SetScript("OnLeave", function() GameTooltip:Hide() end);
  buff.barFrame.buffIcon:SetScript("OnEnter", function() UptimeStatusBarTooltip(buff) end);
  buff.barFrame.buffIcon:SetScript("OnLeave", function() GameTooltip:Hide() end);
end

function UptimeStatusBarTooltip(buff)
  if not InCombatLockdown() then
    GameTooltip:SetOwner(buff.barFrame, "ANCHOR_BOTTOMLEFT");
    GameTooltip:AddLine(buff.name)
    GameTooltip:AddTexture(buff.icon)
    GameTooltip:AddLine((L["Uptime is %s (%s)"]):format(
        buff.barFrame.statusbar.leftText:GetText(),
        buff.barFrame.statusbar.rightText:GetText()
    ))
    GameTooltip:Show()
  else
    GameTooltip:Hide()
  end
end

function UptimeResetButtonTooltip()
  if not InCombatLockdown() then
    GameTooltip:SetOwner(ShamAndAwe.UptimeFrame.ResetButton, "ANCHOR_BOTTOMLEFT");
    GameTooltip:AddLine(L["Reset Session statistics"])
    GameTooltip:Show()
  else
    GameTooltip:Hide()

  end
end

function ShamAndAwe:UpdateUptimeFrames(forceUpdate)
  if InCombatLockdown() or forceUpdate then
    local session = ShamAndAwe.uptime.session
    local lastfight = ShamAndAwe.uptime.lastfight

    --Enhancement
    session.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame.statusbar.leftText:SetText(self:DurationString(session.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].uptime))
    lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame.statusbar.leftText:SetText(self:DurationString(lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].uptime))
    --[[
    --Restoration
    session.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame.statusbar.leftText:SetText(self:DurationString(session.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].uptime))
    lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame.statusbar.leftText:SetText(self:DurationString(lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].uptime))

    session.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame.statusbar.leftText:SetText(self:DurationString(session.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].uptime))
    lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame.statusbar.leftText:SetText(self:DurationString(lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].uptime))
    ]]
    local percent
    if session.totalTime == 0 then
      --Enhancement
      session.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame.statusbar:SetValue(0)
      session.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame.statusbar.rightText:SetText("0%")
      --[[
      --Restoration
      session.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame.statusbar:SetValue(0)
      session.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame.statusbar.rightText:SetText("0%")

      session.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame.statusbar:SetValue(0)
      session.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame.statusbar.rightText:SetText("0%")
      ]]
    else
      --Enhancement
      percent = session.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].uptime / session.totalTime
      session.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame.statusbar:SetValue(percent)
      session.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame.statusbar.rightText:SetText((("%d"):format(percent * 100)) .. "%")
      --[[
      --Restoration
      percent = session.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].uptime / session.totalTime
      session.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame.statusbar:SetValue(percent)
      session.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame.statusbar.rightText:SetText((("%d"):format(percent * 100)) .. "%")

      percent = session.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].uptime / session.totalTime
      session.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame.statusbar:SetValue(percent)
      session.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame.statusbar.rightText:SetText((("%d"):format(percent * 100)) .. "%")
      ]]
    end

    if lastfight.totalTime == 0 then
      --Enhancement
      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame.statusbar:SetValue(0)
      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame.statusbar.rightText:SetText("0%")

      --[[
      --Restoration
      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame.statusbar:SetValue(0)
      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame.statusbar.rightText:SetText("0%")

      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame.statusbar:SetValue(0)
      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame.statusbar.rightText:SetText("0%")
      ]]
    else
      --Enhancement
      percent = lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].uptime / lastfight.totalTime
      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame.statusbar:SetValue(percent)
      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Stormlash ID"])].barFrame.statusbar.rightText:SetText((("%d"):format(percent * 100)) .. "%")

      --[[
      --Restoration
      percent = lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].uptime / lastfight.totalTime
      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame.statusbar:SetValue(percent)
      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Ignore Pain ID"])].barFrame.statusbar.rightText:SetText((("%d"):format(percent * 100)) .. "%")

      percent = lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].uptime / lastfight.totalTime
      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame.statusbar:SetValue(percent)
      lastfight.buffs[GetSpellInfo(ShamAndAwe.constants["Shield Block ID"])].barFrame.statusbar.rightText:SetText((("%d"):format(percent * 100)) .. "%")
      ]]
    end
  end
end

function ShamAndAwe:ResetUptime()
  ShamAndAwe.UptimeFrame:ClearAllPoints()
  ShamAndAwe.db.char.uptime.point = self.defaults.char.uptime.point
  ShamAndAwe.db.char.uptime.relativeTo = self.defaults.char.uptime.relativeTo
  ShamAndAwe.db.char.uptime.relativePoint = self.defaults.char.uptime.relativePoint
  ShamAndAwe.db.char.uptime.xOffset = self.defaults.char.uptime.xOffset
  ShamAndAwe.db.char.uptime.yOffset = self.defaults.char.uptime.yOffset
  ShamAndAwe.db.char.uptime.fWidth = self.defaults.char.uptime.fWidth
  ShamAndAwe.db.char.uptime.fHeight = self.defaults.char.uptime.fHeight
  ShamAndAwe.db.char.uptime.scale = self.defaults.char.uptime.scale
  self:CreateUptimeFrame()
  self:DebugPrint(L["uptime_reset"])
end

function ShamAndAwe:ResetUptimeValues()
  self:InitialiseUptimeBuffs(ShamAndAwe.uptime.session)
  self:InitialiseUptimeBuffs(ShamAndAwe.uptime.lastfight)
  self:UpdateUptimeFrames(true)
end
